use super::{error::PEParseError, header::*};
use std::ops::{Deref, DerefMut};

pub trait PE: Deref<Target = [u8]> + DerefMut<Target = [u8]> + Sized {
    /// Get a reference to a type at a given offset.
    fn get_ref<T>(&self, offset: usize) -> Result<&T, PEParseError> {
        let len = std::mem::size_of::<T>();
        if offset + len > self.len() {
            return Err(PEParseError::OutOfBounds);
        }
        let ptr = self.as_ptr().wrapping_offset(offset as isize) as *const T;
        Ok(unsafe { &*ptr })
    }

    /// Get a mutable reference to a type at a given offset.
    fn get_mut<T>(&mut self, offset: usize) -> Result<&mut T, PEParseError> {
        let len = std::mem::size_of::<T>();
        if offset + len > self.len() {
            return Err(PEParseError::OutOfBounds);
        }
        let ptr = self.as_mut_ptr().wrapping_offset(offset as isize) as *mut T;
        Ok(unsafe { &mut *ptr })
    }

    /// Get the DOS header without verifying its contents.
    fn get_dos_header(&self) -> Result<&ImageDosHeader, PEParseError> {
        let result = self.get_ref::<ImageDosHeader>(0)?;
        Ok(result)
    }
    /// Get the DOS header without mutating its contents.
    fn get_dos_header_mut(&mut self) -> Result<&mut ImageDosHeader, PEParseError> {
        let result = self.get_mut::<ImageDosHeader>(0)?;
        Ok(result)
    }
}
