import styles from "./MainPage.module.scss";
import { useEffect, useState } from "react";
import { Layout, Flex, Button, Empty } from "antd";
import { listen } from "@tauri-apps/api/event";
import { invoke } from "@tauri-apps/api/core";
const { Content } = Layout;
import { SaveOutlined, FolderOpenFilled } from "@ant-design/icons";
import SiderTree from "../components/side_tree/SideTree";
import DosHeader from "../components/DosHeader/DosHeader";
import { open } from "@tauri-apps/plugin-dialog";

const SelectNodeMap = {
  "dos_header": DosHeader,
};

export default function MainPage() {
  const [treeData, setTreeData] = useState([]);
  const [filePath, setFilePath] = useState("");
  const [selectedKey, setSelectedKey] = useState("");
  const [nodeData, setNodeData] = useState([]);

  const openFile = async () => {
    const file = await open({
      multiple: false,
      directory: false,
    });
    console.log("file:", file);
    if (file) {
      setFilePath(file);
    }
  };

  const changeFile = (filePath: string) => {
    invoke("command_open_file", { filePath }).then(() => {
      invoke("command_get_file_pe_node_tree_data").then((res) => {
        console.log("tree nodes:", res);
        setTreeData(res as any);
      });
    });
  };

  const onSelect = (newSelectedKey) => {
    console.log("newSelectedKey", newSelectedKey);
    // TODO: 获取节点数据
    newSelectedKey &&
      invoke(`command_get_pe_data_${newSelectedKey}`).then((resData: any) => {
        console.log("resData", resData);
        // 格式化
        let ary = [];
        for (let key in resData) {
          ary.push(resData[key]);
        }
        console.log("ary", ary);
        setNodeData(ary);
      });
    setSelectedKey(newSelectedKey);
  };

  useEffect(() => {
    const unlisten = listen<{ paths: Array<string> }>(
      "tauri://drag-drop",
      (event) => {
        const filePath = event.payload.paths[0];
        console.log("file path:", filePath);
        setFilePath(filePath);
      }
    );
    return () => {
      unlisten.then((f) => f());
    };
  }, []);

  useEffect(() => {
    if (filePath) {
      changeFile(filePath);
    }
  }, [filePath]);

  console.log("selectedKey", selectedKey);

  return (
    <Layout className={styles.mainLayout}>
      <Flex className={styles.header}>
        <Button
          style={{
            marginRight: "10px",
          }}
          type="dashed"
          onClick={openFile}
          icon={<FolderOpenFilled></FolderOpenFilled>}
        >
          打开新文件
        </Button>
        <Button type="primary" icon={<SaveOutlined></SaveOutlined>}>
          保存
        </Button>
      </Flex>
      <Flex
        style={{
          backgroundColor: "#e5e5e5",
          paddingBottom: "16px",
        }}
      >
        <Flex className={styles.sider}>
          <SiderTree
            treeData={treeData}
            defaultSelectedKey={treeData[0]?.key || ""}
            onSelect={onSelect}
          />
        </Flex>
        <Content>
          {selectedKey && SelectNodeMap[selectedKey] ? (
            SelectNodeMap[selectedKey]({
              defaultData: nodeData,
            })
          ) : (
            <Empty description={"请选择一个节点"} />
          )}
        </Content>
      </Flex>
    </Layout>
  );
}
